import javax.jms.*;

import java.io.*;
import java.util.Properties;

import javax.naming.*;

public class AmbMule implements javax.jms.MessageListener, ExceptionListener {

	static boolean DEBUG = false;
	
	static int DURABLE_MODE = 1;
	static int RELIABLE_MODE = 2;
	static int QUEUE_MODE = 3;
	
	int jmsMode = 0;
	
	private InitialContext jndiContext = null;

	String jndi = null; // "jndi.properties.receive";
    String factoryName = "QueueConnectionFactory";
    String destinationName = null;
    int jmsDeliveryMode = javax.jms.DeliveryMode.PERSISTENT;
    boolean autoAck = true;
    int ackMode = javax.jms.Session.AUTO_ACKNOWLEDGE;
    
    static int msgPriority = 1;
    static int msgTTL = 600000;

            
    boolean testSelector = false;
    boolean isDurable = true;
    String durableID = "myDurableID";

	
	public static String usageError =
		"Usage: java AmbMule [-d|--durable|-r|--reliable] <brokerhost:port> <uid> <pw> {<durableLbPoolName>}";

	public static void main( String[] args ) {
		new AmbMule(args);
	}

	public AmbMule( String[] args ){
		int argc = 0;
		String modeArg = null;
		boolean durable = false;
		String mode = null;
		String broker = null;
		String uid = null;
		String pw = null;
		String appid = "AmbMule";
		String durableLbPoolName = "";

		try {
			modeArg = args[argc++];
			broker = args[argc++];
			uid = args[argc++];
			pw = args[argc++];

			if (argc < args.length) 
				durableLbPoolName = args[argc++];

		} catch( Exception e ) {
			System.out.println(usageError);
			System.exit(1);
		}
		try {
			if (modeArg.matches("-d||--durable")) {
				durable = true;
				mode = "durable";
				jmsMode = DURABLE_MODE;
			} else if (modeArg.matches("-r||--reliable")) {
				durable = false;
				mode = "reliable";
				jmsMode = RELIABLE_MODE;
			} else if (modeArg.matches("-q||--queue")) {
				durable = false;
				mode = "queue";
				jmsMode = QUEUE_MODE;
			} else {
				System.out.println(usageError);
				System.exit(1);
			}

			String pseudoRandom = ((Long)System.currentTimeMillis()).toString();
			pseudoRandom = pseudoRandom.substring(pseudoRandom.length()-7, pseudoRandom.length()-1);
			appid += "-" + mode;
			if (jmsMode != DURABLE_MODE)   // don't create multiple durable subscriptions (to avoid having to delete them later)
				appid += "-" + pseudoRandom;  // make the userid / appId combination unique
			System.out.println("will use application ID: " + appid );
			
			if (DEBUG) System.out.println("Load the context (jndi properties)...");
			getContext();

			Connection connection = null;
			Session session = null;
			MessageProducer producer = null;
			MessageConsumer consumer = null;
			String topicSuffix = (jmsMode == DURABLE_MODE ? "" : "-Reliable");
			
			if (jmsMode == DURABLE_MODE || jmsMode == RELIABLE_MODE ) {
				String propString = "connection.propertyFile=AMB.connection.properties\n";
				propString += "DURABLE_SUBSCRIBER_LB_POOL=" + durableLbPoolName;
				com.u1.jms.TopicConnectionFactory tcf;
				tcf = new com.u1.jms.TopicConnectionFactory(appid, broker, propString);
				connection = tcf.createTopicConnection(uid, pw);
				session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
				Topic inTopic = session.createTopic("muleInTopic" + topicSuffix);
				Topic outTopic = session.createTopic("muleOutTopic" + topicSuffix);
				producer = session.createProducer(inTopic);
				if (durable) {
					consumer = session.createDurableSubscriber(outTopic, "JmsAmbMuleDurableLB");
				} else {
					consumer = session.createConsumer(outTopic);
				}
			} else if (jmsMode == QUEUE_MODE) {
				connection = getQueueConnectionFactory().createConnection();
				session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
				
				Queue outQueue = (Queue) jndiContext.lookup("muleOutQueue");
				if (outQueue == null) { outQueue = session.createQueue("muleOutQueue");}
				consumer = session.createConsumer(outQueue);
				
				Queue inQueue = (Queue) jndiContext.lookup("muleInQueue");
				if (inQueue == null) { inQueue = session.createQueue("muleInQueue");}
				producer = session.createProducer(inQueue);
				producer.setTimeToLive(msgTTL);
				producer.setDeliveryMode(DeliveryMode.PERSISTENT);
			}

			consumer.setMessageListener(this);
			connection.setExceptionListener(this);
			connection.start();
			
			BufferedReader br = new BufferedReader(new InputStreamReader(System.in));

			System.out.println("Running AmbMule with mode: " + mode + "\n");
			for(;;) {

				System.out.print("\n>");
				String line = br.readLine();
				if (line == null) break;

				TextMessage msg = session.createTextMessage(line);
				producer.send(msg);
			}

			if (DEBUG) System.out.println("Clean up...");
			session.close();
			connection.close();

		} catch(Exception e) {
			e.printStackTrace();
		} finally {
		}
	}

	public void onMessage(javax.jms.Message message)
	{	
		try {
			if (message instanceof TextMessage) {
				TextMessage tmsg = (TextMessage) message;
				System.out.println("TextMessage Received: " + tmsg.getText());
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	
	public void onException(javax.jms.JMSException jmse) {
		System.out.println("!!! AmbMule.onException> got exception: " + jmse );
	}

	/** get a JNDI Initial Context.
	 * Loads the jndi.properties file specified in jndi.
	 * 
	 * @return initial context
	 * @throws javax.naming.NamingException
	 */
	private InitialContext getContext() 
		throws javax.naming.NamingException 
	{

		if(jndiContext != null) {
			return jndiContext;
		}
		Properties props = new Properties();

		try {
			if (DEBUG) System.out.println("Will try to load jndi property file = " + jndi);
			if (jndi != null) props.load(new FileInputStream(jndi));
		} catch(IOException e) {
			System.out.println("getContext() IOException: " + e);
		}
		jndiContext = new InitialContext(props);
		return jndiContext;
	}


	public ConnectionFactory getQueueConnectionFactory() {
		ConnectionFactory result = null;
		if (factoryName == null) {
			return null;
		} try {
			if (DEBUG) System.out.println("Looking up factoryName = " + factoryName );
			result = (ConnectionFactory) getContext().lookup(factoryName);
		} catch (javax.naming.NamingException e) {
			if (DEBUG) System.out.println("getConnectionFactory() NamingException for " + factoryName);
			e.printStackTrace();
		}
		if (DEBUG) System.out.println("getConnectionFactory " + factoryName + " returning a " + result.getClass().getName());

		return result;
	}
	
}
